@@ -3,7 +3,7 @@
 # Address verification functions
 #
 # This file is part of SAUCE, a very picky anti-spam receiver-SMTP.
-# SAUCE is Copyright (C) 1997-2001 Ian Jackson
+# SAUCE is Copyright (C) 1997-2003 Ian Jackson
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -19,7 +19,7 @@
 # along with this program; if not, write to the Free Software Foundation,
 # Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. 
 #
-# $Id: avf.tcl,v 1.21 2001/03/21 23:53:14 ian Exp $
+# $Id: avf.tcl,v 1.23 2003/06/15 15:46:40 ian Exp $
 
 
 ########## avf threads
@@ -312,7 +312,8 @@ thread_subproc avf result {ok text} {
     thread_finish avf $id $ok $text
 }
 
-thread_chainproc avf vc_err {emsg} {
+thread_chainproc avf vc_err {args} {
+    set emsg [lindex $args end]
     avf_tempfail $state(conncurrentlylev) "\[$state(caddr)\] $emsg"
     avf_vc_failed
 }
