@@ -18,7 +18,7 @@
 # along with this program; if not, write to the Free Software Foundation,
 # Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. 
 #
-# $Id: msgdata.tcl,v 1.40 2003/06/15 15:46:40 ian Exp $
+# $Id: msgdata.tcl,v 1.47 2003/12/29 17:23:51 ian Exp $
 
 # While receiving the message data, we are either:
 # * Receiving the header:
@@ -148,7 +148,7 @@ thread_subproc ic hdr_bad {eom problem} {
 thread_subproc ic msg_checkeof {} {
     global canonical_hostname
     if {[eof $state(chan)]} {
-	catch_close_cleardesc $state(mtachan)
+	catch_close_cleardesc state(mtachan)
 	ic_commandfinalresponse major \
 		"421 $canonical_hostname Connection dropped in message data"
 	return -code return
@@ -413,14 +413,21 @@ thread_chainproc ic body_data_ok {eom rejwarnings data} {
 	foreach w $rejwarnings {
 	    append hdrdata "X-SAUCE-Warning: ($canonical_hostname) $w\n"
 	}
+	manyset [ic_getsiteannoy 0] annoyval annoystr
+	if {$annoyval > 0} {
+	    append hdrdata \
+		    "X-SAUCE-Notice: ($canonical_hostname) $annoystr" \
+		    " ${annoyval}ms (delays likely)\n" \
+		    "                with $state(rh) \[$state(ra)\]\n"
+	}
     }
     if {$add_received} {
-	regsub {^.Name\: } {$Name: debian_version_0_7_16 $} {} rcsinfo
+	regsub {^.Name\: } {$Name: debian_version_0_8_1 $} {} rcsinfo
 	if {![regexp {^[ $]*$} $rcsinfo]} {
 	    regsub {^debian_version_} $rcsinfo v rcsinfo
 	    regsub -all _ $rcsinfo . rcsinfo
 	} else {
-	    regsub {^.Revision\: } {$Revision: 1.40 $} r rcsinfo
+	    regsub {^.Revision\: } {$Revision: 1.47 $} r rcsinfo
 	}
 	regsub {[ $]*$} $rcsinfo {} rcsinfo
 	set date [date_822]
@@ -447,15 +454,49 @@ thread_chainproc ic body_copy {eom} {
 }
 
 thread_chainproc ic body_read {data} {
+    global canonical_hostname taboo_virus_hack
     ic_msg_checkeof
-    if {"$data" == "."} {
+    if {![string compare . $data]} {
 	ic_body_eom
+    } elseif {$taboo_virus_hack && [string match \
+*TVqQAAMAAAAEAAAA//8AALgAAAAAAAAAQAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA*\
+ $data]} {
+        ic_bodybodyrej "554 virus flood (or bare executable) rejected"
     } else {
 	threadio_puts ic $id $state(mtachan) "$data\n" body_copy {} 0
     }
 }
 
-# Now we have final dot
+thread_subproc ic bodybodyrej {resp} {
+    # Used for rejecting the message body after the header
+    # has been read (ie, when we're already feeding the MTA data).
+    catch_close_cleardesc state(mtachan)
+    ic_bodyrej_startread [list ic_bodybodyrej_done $resp]
+}
+
+thread_subproc ic bodybodyrej_done {resp} {
+    global annoyance_major
+    chanset_hide $state(chan) 1 1
+
+    set annoychange $annoyance_major
+    set irritvalx {}
+    if {[info exists state(rcpts_errok_only)] && \
+	    $state(rcpts_errok_only)} {
+	set annoychange 0
+	set irritvalx -
+    }
+    manyset [ic_getsiteannoy $annoychange] irritval irritamt
+    set irritamtl [string tolower $irritamt]
+    
+    logreject reject state command $resp $irritamtl "$irritval$irritvalx"
+    
+    threadio_puts ic $id $state(chan) "$resp\n" bodybodyrej_rejdone {}
+}
+
+thread_chainproc ic bodybodyrej_rejdone {} {
+    ic_msg_resetvars
+    ic_mtachan_open
+}
 
 thread_subproc ic body_eom {} {
     chanset_hide $state(mtachan) 1 1
@@ -545,19 +586,23 @@ thread_subproc ic body_minfo {mid} {
 thread_subproc ic bodyrej {eom why} {
     ic_msg_resethdrvars
     if {!$eom} {
-	threadio_gets ic $id $state(chan) bodyrej_read {} $why
+	ic_bodyrej_startread [list ic_bodyrej 1 $why]
     } else {
 	threadio_commandresponse ic $id $state(mtachan) rset {^2..} \
 		bodyrej_resetmta_ok {} $why
     }
 }
 
-thread_chainproc ic bodyrej_read {why data} {
+thread_subproc ic bodyrej_startread {then} {
+    threadio_gets ic $id $state(chan) bodyrej_read {} $then
+}
+
+thread_chainproc ic bodyrej_read {then data} {
     ic_msg_checkeof
     if {"$data" == "."} {
-	ic_bodyrej 1 $why
+	eval $then
     } else {
-	threadio_gets ic $id $state(chan) bodyrej_read {} $why
+	threadio_gets ic $id $state(chan) bodyrej_read {} $then
     }
 }
 
@@ -584,7 +629,7 @@ thread_subproc ic blacklist_thing {sa value tellwho valdesc} {
 	    log notice "$cd ... $sa $value ... blacklisting"
 	    set thread [thread_start notifybl \
 		    "$state(desc) / notify-bl $sa $value" \
-		    $tellwho $valdesc $state(add_bl)]
+		    $tellwho $sa $valdesc $state(add_bl)]
 	    thread_join {} {} notifybl $thread addbl_done addbl_err \
 		    $state(desc) $tellwho "$sa $value"
 	    setstate $sa $value $add_bl black $bl_to
@@ -660,7 +705,7 @@ proc addbl_err {desc what where msg} {
 thread_subproc ic msg_resetvars {} {
     catch { thread_cancel avf $state(avfid) }
     ic_msg_resethdrvars
-    foreach x {mf_lp mf_dm mf_parms mid hdrscomplete} {
+    foreach x {mf_lp mf_dm mf_parms mid hdrscomplete rcpts_errok_only} {
 	catch { unset state($x) }
     }
     foreach x {rej_msg defer_msg delay_msg att_rcpts a_kinds resentmids} {
